<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet SYSTEM "MathEntities.dtd">
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:msxsl="urn:schemas-microsoft-com:xslt"
	xmlns:csharp-code="grac:c#-code">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

	<xsl:variable name="math_ns" select="'http://www.w3.org/1998/Math/MathML'"/>

	<xsl:template match="math">
		<!-- add w3org math namespace for validation with schema later -->
		<xsl:element name="math" namespace="{$math_ns}">
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>
	
	<!-- containes the main logic of row(real or inferred) processing-->
	<xsl:template name="row_processor">
		<xsl:choose>
			<xsl:when test="count(mo) > 0"> <!-- the row contains mo elements immediately -->
				<xsl:apply-templates select="mo[1]"/> 
				<!-- we should apply the template for the first mo in the mrow because the other mo will be processed automatically-->
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="mrow">
		<xsl:choose>
			<xsl:when test="count(*) = 1"> <!-- do not copy mrow in this case  -->
				<xsl:call-template name="row_processor"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="mrow" namespace="{$math_ns}">
					<xsl:call-template name="row_processor"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="mstyle">
		<xsl:choose>
			<xsl:when test="count(*) = 1"> <!-- do not copy mrow in this case  -->
				<xsl:call-template name="row_processor"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="mrow" namespace="{$math_ns}">
					<xsl:call-template name="row_processor"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<xsl:template match="inferred_row">
		<xsl:call-template name="row_processor"/>
	</xsl:template>

	<xsl:template name="writeRow">
		<xsl:param name="row_content"/>
		<xsl:copy-of select="$row_content"/>
		<!--xsl:choose>
			<xsl:when test="count($row_content) = 1">
				<xsl:copy-of select="$row_content"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="mrow" namespace="{$math_ns}">
					<xsl:copy-of select="$row_content"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose-->
	</xsl:template>

	<xsl:template name="inferRowFor">
		<xsl:param name="it"/>
		<xsl:variable name="inferred_row">
			<xsl:choose>
				<xsl:when test="count($it) = 1"> <!-- do not create the new row for the node set of lenght = 1 -->
					<xsl:copy-of select="$it"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:element name="inferred_row">
						<xsl:copy-of select="$it"/>
					</xsl:element>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:apply-templates select="msxsl:node-set($inferred_row)"/>
	</xsl:template>

	<xsl:template name="inferRowForLeftHandOperand">
		<xsl:call-template name="inferRowFor">
			<xsl:with-param name="it" select="preceding-sibling::*"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="inferRowForRightHandOperand">
		<xsl:param name="operator_attrs"/>
		<xsl:variable name="inferred_row_content">
			<xsl:call-template name="inferRowFor">
				<xsl:with-param name="it" select="following-sibling::*"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="inferred_row_content_ns" select="msxsl:node-set($inferred_row_content)"/>
		<xsl:variable name="inferred_row_first_mo_form_value" select="$inferred_row_content_ns/*[local-name() = 'mo'][1]/@form"/>
		<xsl:variable name="inferred_row_first_mo_group_number" select="$inferred_row_content_ns/*[local-name() = 'mo'][1]/@group_number"/>
		<xsl:variable name="inferred_row_first_mo_fence" select="$inferred_row_content_ns/*[local-name() = 'mo'][1]/@fence"/>

		<xsl:choose>
			<xsl:when test="($inferred_row_first_mo_fence='true') and ($inferred_row_first_mo_form_value='postfix')">
				<xsl:copy-of select="$inferred_row_content_ns"/>
			</xsl:when>
			<xsl:when test="(($operator_attrs/@form='prefix') or ($operator_attrs/@form='infix')) and
								  (($inferred_row_first_mo_form_value='infix') or ($inferred_row_first_mo_form_value='postfix')) and
								  (number($inferred_row_first_mo_group_number) = number($operator_attrs/@group_number))">
				<xsl:copy-of select="$inferred_row_content_ns"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeRow">
					<xsl:with-param name="row_content" select="$inferred_row_content_ns/*"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:if test="$inferred_row_first_mo_form_value='postfix'">
			<xsl:variable name="row_content">
				<postfix_op_stub/><!-- we use this stub to the next operator could have the infix form  -->
				<xsl:copy-of select="following-sibling::mo[1]/following-sibling::*"/><!-- we use this query because the postfix operator writes itself and stoppes the writting -->
			</xsl:variable>
			<xsl:call-template name="inferRowFor">
				<xsl:with-param name="it" select="msxsl:node-set($row_content)/*"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="writeOperatorItself">
		<xsl:param name="operator_attrs"/>
		<xsl:element name="mo" namespace="{$math_ns}">
			<xsl:copy-of select="$operator_attrs/@form"/>
			<xsl:copy-of select="$operator_attrs/@fence"/>
			<xsl:copy-of select="$operator_attrs/@separator"/>
			<xsl:copy-of select="$operator_attrs/@accent"/>
			<xsl:copy-of select="$operator_attrs/@group_number"/>
			<xsl:value-of select="$operator_attrs/@value"/>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writePrefixOperator">
		<xsl:param name="operator_attrs"/>
		<xsl:call-template name="inferRowForLeftHandOperand"/>
		<xsl:call-template name="writeOperatorItself">
			<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
		</xsl:call-template>
		<xsl:call-template name="inferRowForRightHandOperand">
			<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="writeInfixOperator">
		<xsl:param name="operator_attrs"/>
		<xsl:call-template name="inferRowForLeftHandOperand"/>
		<xsl:call-template name="writeOperatorItself">
			<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
		</xsl:call-template>
		<xsl:call-template name="inferRowForRightHandOperand">
			<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="writePostfixOperator">
		<xsl:param name="operator_attrs"/>
		<xsl:call-template name="inferRowForLeftHandOperand"/>
		<xsl:call-template name="writeOperatorItself">
			<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="writeOperatorWithItsOperands">
		<xsl:param name="operator_attrs"/>
		<xsl:variable name="op_form" select="$operator_attrs/@form"/>
		<xsl:choose>
			<xsl:when test="$op_form='prefix'">
				<xsl:call-template name="writePrefixOperator">
					<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$op_form='infix'">
				<xsl:call-template name="writeInfixOperator">
					<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$op_form='postfix'">
				<xsl:call-template name="writePostfixOperator">
					<xsl:with-param name="operator_attrs" select="$operator_attrs"/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:template>
	<!-- 
	csharp-code:qualifyOperator(.) method returns a single element as the next is:
	<entity value='[mo value]' form={'prefix', 'infix', 'postfix'} fence={'true', 'false'} separator={'true', 'false'} accent={'true', 'false'}/>.
	It is the result of analysis of mo element which is received as the parameter for the method.
	So we can perform the processing below.
	-->
	<xsl:template match="mo">
	
	
	<!--xsl:if test='text()="&apos;"'>
	<xsl:if test="text()='&gamma;'">
		<xsl:variable name="a">mama &gamma; papa</xsl:variable>
		<mycode>
		<xsl:value-of select="text()"></xsl:value-of>
		<xsl:value-of select="$a"></xsl:value-of>
		</mycode>
	</xsl:if-->
	
	
	
		<!--xsl:choose>
			<xsl:when test="count(parent::mrow/mo) = 1"> < do not create unnecessary mrow >
				<xsl:call-template name="writeOperatorWithItsOperands">
					<xsl:with-param name="operator_attrs" select="csharp-code:qualifyOperator(.)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise> < put an operator and all its operands into the single row. >
				<xsl:element name="mrow" namespace="{$math_ns}">
					<xsl:call-template name="writeOperatorWithItsOperands">
						<xsl:with-param name="operator_attrs" select="csharp-code:qualifyOperator(.)"/>
					</xsl:call-template>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose-->
			<xsl:call-template name="writeOperatorWithItsOperands">
				<xsl:with-param name="operator_attrs" select="csharp-code:qualifyOperator(.)"/>
			</xsl:call-template>
	</xsl:template>

	<xsl:template match="postfix_op_stub"/><!-- inferred element -->

	<xsl:template name="putAllChildrenIntoRow">
		<xsl:for-each select="*">
			<xsl:choose>
				<xsl:when test="local-name(current()) = 'mrow'">
					<xsl:apply-templates select="current()"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="wrapped_child">
						<xsl:element name="mrow">
							<xsl:copy-of select="current()"/>
						</xsl:element>
					</xsl:variable>
					<xsl:apply-templates select="msxsl:node-set($wrapped_child)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="writeComplexEntity">
		<xsl:element name="{local-name()}" namespace="{$math_ns}">
			<xsl:call-template name="putAllChildrenIntoRow"/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="mfrac">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="mroot">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="msqrt">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="msub">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="msup">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="msubsup">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="munder">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="mover">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="munderover">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="mtable">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="mtr">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>

	<xsl:template match="mtd">
		<xsl:call-template name="writeComplexEntity"/>
	</xsl:template>
	
	<xsl:template name="createNumber">
		<xsl:param name="number"/>
		<xsl:element name="mn" namespace="{$math_ns}">
			<xsl:value-of select="$number"/>
		</xsl:element>
	</xsl:template>
	<!-- extract the dots from a number (like 0.66... and ...55).
		create 2 elements <mn> and <mo>...</mo>	 -->
	<xsl:template match="mn">
		<xsl:choose>
			<xsl:when test="csharp-code:isNumberPrefixedByDots(text())">
				<xsl:variable name="dots">
						<entry form='prefix' value='...'/>
				</xsl:variable>
				<xsl:call-template name="writeOperatorItself">
					<xsl:with-param name="operator_attrs" select="msxsl:node-set($dots)/*"/>
				</xsl:call-template>
				<xsl:call-template name="createNumber">
					<xsl:with-param name="number" select="substring(text(), 4)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="csharp-code:isNumberPostfixedByDots(text())">
				<xsl:call-template name="createNumber">
					<xsl:with-param name="number" select="substring(text(), 1, string-length(text()) - 3)"/>
				</xsl:call-template>
				<xsl:variable name="dots">
					<entry form='postfix' value='...'/>
				</xsl:variable>
				<xsl:call-template name="writeOperatorItself">
					<xsl:with-param name="operator_attrs" select="msxsl:node-set($dots)/*"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="createNumber">
					<xsl:with-param name="number" select="text()"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*">
		<xsl:element name="{local-name()}"  namespace="{$math_ns}">
			<xsl:copy-of select="@*"/>
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="text()">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>





</xsl:stylesheet>
