<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
		xmlns:csharp-code="grac:c#-code"
		xmlns:formatting="grac:formatting"
		exclude-result-prefixes="formatting csharp-code msxsl">

	<!-- include rules for inserting formattind commands -->
	<xsl:import href="FormattingCmdInserter.xslt"/>

	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

	<xsl:template name="leaveTrace">
		<xsl:param name="entity"/>
		<xsl:copy>
			<xsl:copy-of select="@*"/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template name="createFlowAtom">
		<xsl:param name="entity"/>
		<xsl:param name="content"/>
		<!-- create a flow atom if content is not empty -->
		<xsl:if test="count(msxsl:node-set($content)/*) > 0">
			<flow_atom>
				<!-- reflect the entity for which the flow atom is creating -->
				<xsl:call-template name="leaveTrace">
					<xsl:with-param name="entity">
						<xsl:choose>
							<xsl:when test="$entity"><!-- entity is pointed -->
								<xsl:copy-of select="$entity"/>
							</xsl:when>
							<xsl:otherwise><!-- get the current entity -->
								<xsl:copy-of select="."/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:with-param>
				</xsl:call-template>
				<!-- add content -->
				<xsl:copy-of select="$content"/>
			</flow_atom>
		</xsl:if>
	</xsl:template>

	<xsl:template name="copyItself">
		<xsl:param name="entity"/>
		<xsl:variable name="entity_to_copy">
			<xsl:choose>
				<xsl:when test="$entity">
					<xsl:copy-of select="$entity"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:copy-of select="."/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="{local-name($entity_to_copy)}">
			<xsl:copy-of select="@*"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="/">
		<result>
			<xsl:apply-templates select="//expression"/>
		</result>
	</xsl:template>

	<xsl:template match="expression">
		<!--xsl:comment>expression start</xsl:comment-->
		<expression>
			<xsl:apply-templates/>
		</expression>
		<!--xsl:comment>expression end</xsl:comment-->
	</xsl:template>

	<xsl:template match="*">
		<!-- write the qualifiers in first -->
		<xsl:apply-templates select="qualifier"/>
		<!-- if there is a template in the complex entities templates list than try to use the template -->
		<xsl:call-template name="useTemplate"/>
	</xsl:template>

	<xsl:template name="traceFTemplate">
		<xsl:param name="formatting_template"/>
		<xsl:comment>formatting_template was found<xsl:value-of select="local-name($formatting_template)"/> name = <xsl:value-of select="$formatting_template/@name"/></xsl:comment>
	</xsl:template>
		
	<xsl:template name="writeStartingFormattingRulesForComplexEntity">
		<xsl:param name="formatting_template"/>
		<xsl:call-template name="traceFTemplate">
			<xsl:with-param name="formatting_template" select="$formatting_template"/>
		</xsl:call-template>
		<xsl:apply-templates select="$formatting_template/break_before" mode="for_composite_entity"/>
		<xsl:apply-templates select="$formatting_template/@hier_decrement" mode="for_composite_entity"/>
		<xsl:apply-templates select="$formatting_template/@can_ltrim" mode="for_composite_entity"/>
		<xsl:apply-templates select="$formatting_template/break" mode="for_composite_entity"/>
	</xsl:template>

	<xsl:template name="writeEndingFormattingRulesForComplexEntity">
		<xsl:param name="formatting_template"/>
		<xsl:call-template name="traceFTemplate">
			<xsl:with-param name="formatting_template" select="$formatting_template"/>
		</xsl:call-template>
		<xsl:apply-templates select="$formatting_template/break_after" mode="for_composite_entity"/>
	</xsl:template>

	<!--xsl:template name="writeBeforeSequence">
		<xsl:param name="decoding_template"/>
		<xsl:comment><xsl:value-of select="concat('template found context[', $decoding_template/@context, ']')"/></xsl:comment>
		<flow_atom>
			<xsl:apply-templates select="$decoding_template/before_seq"/>
		</flow_atom>
	</xsl:template>

	<xsl:template name="writeAfterSequence">
		<xsl:param name="decoding_template"/>
		<flow_atom>
			<xsl:apply-templates select="$decoding_template/after_seq"/>
		</flow_atom>
	</xsl:template-->
	<xsl:template name="writeBeforeSequence">
		<xsl:param name="dtemplate_content"/>
		<flow_atom>
			<xsl:apply-templates select="$dtemplate_content/before_seq"/>
		</flow_atom>
	</xsl:template>

	<xsl:template name="writeAfterSequence">
		<xsl:param name="dtemplate_content"/>
		<flow_atom>
			<xsl:apply-templates select="$dtemplate_content/after_seq"/>
		</flow_atom>
	</xsl:template>
	
	<xsl:template name="useTemplate">
		<xsl:variable name="appropriate_template" select="csharp-code:getTheMostMatchingTemplate(.)"/>
		<xsl:if test="count($appropriate_template) > 0"> <!-- do the job if some decoding template was found -->
			<xsl:variable name="dtemplate_content" select="csharp-code:findContentOwnerForTemplate($appropriate_template)"/>
			<!-- find out the most matching formating template -->
			<xsl:variable name="formatting_template" select="formatting:getFormattingTemplate($appropriate_template, .)"/>
			<xsl:copy>
				<xsl:copy-of select="@*"/>

				<xsl:call-template name="writeStartingFormattingRulesForComplexEntity">
					<xsl:with-param name="formatting_template" select="$formatting_template"/>
				</xsl:call-template>

				<xsl:comment><xsl:value-of select="concat('template found context[', $appropriate_template/@context, ']')"/></xsl:comment>
				<xsl:call-template name="writeBeforeSequence">
					<xsl:with-param name="dtemplate_content" select="$dtemplate_content"/>
				</xsl:call-template>

				<xsl:choose>
					<xsl:when test="$appropriate_template/Table">
						<xsl:value-of select="formatting:pushFormattingTemplate($formatting_template)"/>

						<xsl:call-template name="useTemplateForChildren">
							<xsl:with-param name="template" select="$appropriate_template"/>
						</xsl:call-template>

						<xsl:value-of select="formatting:popFormattingTemplate()"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:variable name="ordered_list" select="csharp-code:getOrderedList(., $dtemplate_content)"/>
						<!-- check there is a common separator for the collection -->
						<xsl:variable name="common_separator" select="$dtemplate_content/separator[@type='common']"/>
						<xsl:for-each select="$ordered_list/self::node()">
							<xsl:variable name="child_pos" select="position()"/>
							<!-- try to write a separator -->
							<xsl:if test=" ($child_pos &gt; 1)">
								<xsl:choose>
									<xsl:when test="$common_separator"> <!-- we have the common separator -->
										<xsl:apply-templates select="$common_separator"/>
									</xsl:when>
									<xsl:otherwise> <!-- find a 'manual' separator -->
										<xsl:apply-templates select="$dtemplate_content/separator[$child_pos - 1]"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:if>
							<xsl:value-of select="formatting:pushFormattingTemplate($formatting_template)"/>
							<!-- process sub entity(it can be a composite entity or a simple entity ) -->
							<xsl:apply-templates select="current()"/>
							<xsl:value-of select="formatting:popFormattingTemplate()"/>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>

				<xsl:call-template name="writeAfterSequence">
					<xsl:with-param name="dtemplate_content" select="$dtemplate_content"/>
				</xsl:call-template>

				<xsl:call-template name="writeEndingFormattingRulesForComplexEntity">
					<xsl:with-param name="formatting_template" select="$formatting_template"/>
				</xsl:call-template>
			</xsl:copy>
		</xsl:if>
	</xsl:template>

	<xsl:template name="useTemplateForChildren">
		<xsl:param name="template"/>
		<xsl:for-each select="*">
			<xsl:element name="{local-name(current())}">
				<xsl:copy-of select="@*"/>
				<flow_atom>
					<xsl:apply-templates select="csharp-code:decodeByDictionaryEx(current(), $template)"/>
				</flow_atom>
			</xsl:element>
		</xsl:for-each>
	</xsl:template>

	<!-- process entities from BrailleEntityTemplate.xml -->
	<xsl:template match="before_seq">
		<xsl:comment>start to write [before_seq]</xsl:comment>
		<xsl:apply-templates/>
		<xsl:comment>end to write [before_seq]</xsl:comment>
	</xsl:template>
	<xsl:template match="after_seq">
		<xsl:comment>start to write [after_seq]</xsl:comment>
		<xsl:apply-templates/>
		<xsl:comment>end to write [after_seq]</xsl:comment>
	</xsl:template>
	<xsl:template match="separator">
		<xsl:comment>start to write [separator]</xsl:comment>
		<flow_atom>
			<xsl:apply-templates/>
		</flow_atom>
		<xsl:comment>end to write [separator]</xsl:comment>
	</xsl:template>
	<!-- resolve link -->
	<xsl:template match="use"><!-- allows to use a link to the BrailleDictionary.xml//GeneralBrailleEntities table -->
		<xsl:apply-templates select="csharp-code:resolveLink(@entity)"/>
	</xsl:template>
	<!--  process entities from BrailleDictionary.xml -->
	<xsl:template match="br_atom">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<!-- processing of the qualifiers -->
	<xsl:template match="qualifier">
		<xsl:comment>start to write [qualifier]</xsl:comment>
		<xsl:copy>
			<xsl:copy-of select="@ref"/>
			<xsl:apply-templates select="put"/>
		</xsl:copy>
		<xsl:comment>end to write [qualifier]</xsl:comment>
	</xsl:template>

	<xsl:template match="qualifier/put">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="qualifier[@type='obligatory']">
		<xsl:comment>start to write [obligatory qualifier]</xsl:comment>
			<xsl:apply-templates select="put/*"/>
		<xsl:comment>end to write [obligatory qualifier]</xsl:comment>
	</xsl:template>

</xsl:stylesheet>